<?php

namespace Litematrimony\Profile\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litematrimony\Profile\Http\Requests\RequestRequest;
use Litematrimony\Profile\Interfaces\RequestRepositoryInterface;
use Litematrimony\Profile\Models\Request;

/**
 * Resource controller class for request.
 */
class RequestResourceController extends BaseController
{

    /**
     * Initialize request resource controller.
     *
     * @param type RequestRepositoryInterface $request
     *
     * @return null
     */
    public function __construct(RequestRepositoryInterface $request)
    {
        parent::__construct();
        $this->repository = $request;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litematrimony\Profile\Repositories\Criteria\RequestResourceCriteria::class);
    }

    /**
     * Display a list of request.
     *
     * @return Response
     */
    public function index(RequestRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litematrimony\Profile\Repositories\Presenter\RequestPresenter::class)
                ->$function();
        }

        $requests = $this->repository->paginate();

        return $this->response->title(trans('profile::request.names'))
            ->view('profile::request.index', true)
            ->data(compact('requests'))
            ->output();
    }

    /**
     * Display request.
     *
     * @param Request $request
     * @param Model   $request
     *
     * @return Response
     */
    public function show(RequestRequest $request, Request $request)
    {

        if ($request->exists) {
            $view = 'profile::request.show';
        } else {
            $view = 'profile::request.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('profile::request.name'))
            ->data(compact('request'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new request.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(RequestRequest $request)
    {

        $request = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('profile::request.name')) 
            ->view('profile::request.create', true) 
            ->data(compact('request'))
            ->output();
    }

    /**
     * Create new request.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(RequestRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $request                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('profile::request.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('profile/request/' . $request->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/profile/request'))
                ->redirect();
        }

    }

    /**
     * Show request for editing.
     *
     * @param Request $request
     * @param Model   $request
     *
     * @return Response
     */
    public function edit(RequestRequest $request, Request $request)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('profile::request.name'))
            ->view('profile::request.edit', true)
            ->data(compact('request'))
            ->output();
    }

    /**
     * Update the request.
     *
     * @param Request $request
     * @param Model   $request
     *
     * @return Response
     */
    public function update(RequestRequest $request, Request $request)
    {
        try {
            $attributes = $request->all();

            $request->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('profile::request.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('profile/request/' . $request->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('profile/request/' . $request->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the request.
     *
     * @param Model   $request
     *
     * @return Response
     */
    public function destroy(RequestRequest $request, Request $request)
    {
        try {

            $request->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('profile::request.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('profile/request/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('profile/request/' . $request->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple request.
     *
     * @param Model   $request
     *
     * @return Response
     */
    public function delete(RequestRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('profile::request.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('profile/request'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/profile/request'))
                ->redirect();
        }

    }

    /**
     * Restore deleted requests.
     *
     * @param Model   $request
     *
     * @return Response
     */
    public function restore(RequestRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('profile::request.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/profile/request'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/profile/request/'))
                ->redirect();
        }

    }

}
